/*************************************************************************************************
Dmitri Korotkevitch
http://aboutsqlserver.com
email: dmitri@aboutsqlserver.com

24HOP Russia (2015) - Size does Matter
06.Monitoring Page Splits with xEvents (SQL Server 2012+)

Based on Jonathan Kehayias' blog post: 
https://www.sqlskills.com/blogs/jonathan/tracking-problematic-pages-splits-in-sql-server-2012-extended-events-no-really-this-time/
*************************************************************************************************/
use master
go

-- Getting database id of our database
select * from sys.databases where name = '24HOP_2015'
go

if exists(select * from sys.server_event_sessions where name = 'PageSplits_Tracking')
	drop event session PageSplits_Tracking on server
go

create event session PageSplits_Tracking
on server
add event sqlserver.transaction_log
(
    where operation = 11  -- lop_delete_split 
		and database_id = 17 
)
add target package0.histogram
(
    set 
		filtering_event_name = 'sqlserver.transaction_log',
        source_type = 0, -- event column
        source = 'alloc_unit_id'
)
go
        
-- start the event session
alter event session PageSplits_Tracking
on server
state=start;
go

use [24hop_2015]
go

-- We just rebuilt index with FILLFACTOR=100
if object_id('tempdb..#TempUID') is not null drop table #TempUID
create table #TempUID(ID uniqueidentifier default NEWID(), Num int)
go

declare
	@I int = 201 -- Change during execution

truncate table #TempUID
	 
;with n1(c) as (select 0 union all select 0) -- 2 rows
,n2(c) as (select 0 from n1 as t1 cross join n1 as t2) -- 4 rows
,n3(c) as (select 0 from n2 as t1 cross join n2 as t2) -- 16 rows
,n4(c) as (select 0 from n3 as t1 cross join n3 as t2) -- 256 rows
,n5(c) as (select 0 from n4 as t1 cross join n4 as t2) -- 65,536 rows
,ids(id) as (select row_number() over (order by (select null)) from n5)
	insert into #TempUID(Num) 
	select ID from Ids

insert into dbo.Customers(CustomerId, CustomerNumber, CustomerName, CreditLimit, Attributes)
	select ID 
		,convert(nvarchar(16),@I * 65536 + 1 + Num)
		,N'Customer ' + convert(nvarchar(48),ID)
		,Abs(CheckSum(ID))
		,null
	from #TempUID
go

-- Query Target Data to get the top splitting objects in the database:
;with Data(alloc_unit_id, splits)
as
(
	select 
		c.n.value('(value)[1]', 'bigint') as alloc_unit_id
        ,c.n.value('(@count)[1]', 'bigint') as splits
	from
		(
			select convert(xml,target_data) target_data
			from 
				sys.dm_xe_sessions s with (nolock)
					join sys.dm_xe_session_targets t on
					s.address = t.event_session_address
			where 
				s.name = 'PageSplits_Tracking' and t.target_name = 'histogram' 
		) as d
        cross apply 
			target_data.nodes('HistogramTarget/Slot') as c(n)
)
select
	s.name + '.' + o.name as [Table]
    ,i.index_id 
	,i.name as [Index]
    ,d.Splits
    ,i.fill_factor as [Fill Factor]
from 
	Data d join sys.allocation_units au with (nolock) on
		d.alloc_unit_id = au.allocation_unit_id
	join sys.partitions p with (nolock) on
		au.container_id = p.partition_id
	join sys.indexes i with (nolock) on 
		p.object_id = i.object_id and p.index_id = i.index_id
	join sys.objects o with (nolock) on
		i.object_id = o.object_id
	join sys.schemas s on
		o.schema_id = s.schema_id
where
	o.is_ms_shipped = 0
go

alter event session PageSplits_Tracking on server state=stop;
go

-- Check Total Number of Pages/Fragmentation before rebuild

alter index IDX_Customers_CustomerID on dbo.Customers rebuild with (fillfactor=80)
go

-- Cleanup Target

alter event session PageSplits_Tracking on server state=start;
go

declare
	@I int = 202 -- Change during execution

truncate table #TempUID
	 
;with n1(c) as (select 0 union all select 0) -- 2 rows
,n2(c) as (select 0 from n1 as t1 cross join n1 as t2) -- 4 rows
,n3(c) as (select 0 from n2 as t1 cross join n2 as t2) -- 16 rows
,n4(c) as (select 0 from n3 as t1 cross join n3 as t2) -- 256 rows
,n5(c) as (select 0 from n4 as t1 cross join n4 as t2) -- 65,536 rows
,ids(id) as (select row_number() over (order by (select null)) from n5)
	insert into #TempUID(Num) 
	select ID from Ids

insert into dbo.Customers(CustomerId, CustomerNumber, CustomerName, CreditLimit, Attributes)
	select ID 
		,convert(nvarchar(16),@I * 65536 + 1 + Num)
		,N'Customer ' + convert(nvarchar(48),ID)
		,Abs(CheckSum(ID))
		,null
	from #TempUID
go

-- Query Target Data to get the top splitting objects in the database:
;with Data(alloc_unit_id, splits)
as
(
	select 
		c.n.value('(value)[1]', 'bigint') as alloc_unit_id
        ,c.n.value('(@count)[1]', 'bigint') as splits
	from
		(
			select convert(xml,target_data) target_data
			from 
				sys.dm_xe_sessions s with (nolock)
					join sys.dm_xe_session_targets t on
					s.address = t.event_session_address
			where 
				s.name = 'PageSplits_Tracking' and t.target_name = 'histogram' 
		) as d
        cross apply 
			target_data.nodes('HistogramTarget/Slot') as c(n)
)
select
	s.name + '.' + o.name as [Table]
    ,i.index_id 
	,i.name as [Index]
    ,d.Splits
    ,i.fill_factor as [Fill Factor]
from 
	Data d join sys.allocation_units au with (nolock) on
		d.alloc_unit_id = au.allocation_unit_id
	join sys.partitions p with (nolock) on
		au.container_id = p.partition_id
	join sys.indexes i with (nolock) on 
		p.object_id = i.object_id and p.index_id = i.index_id
	join sys.objects o with (nolock) on
		i.object_id = o.object_id
	join sys.schemas s on
		o.schema_id = s.schema_id
where
	o.is_ms_shipped = 0
go

alter event session PageSplits_Tracking on server state=stop;
go