/*************************************************************************************************
Dmitri Korotkevitch
http://aboutsqlserver.com
email: dmitri@aboutsqlserver.com

24HOP Russia (2015) - Size does Matter
07.Data Compression
*************************************************************************************************/
USE [tempdb]
GO

CREATE TABLE [dbo].[data]
(
	[RECID] [bigint] NOT NULL,
	[COMPANYID] [int] NOT NULL,
	[USERID] [int] NOT NULL,
	[MSGVERSION] [tinyint] NOT NULL,
	[SEQVALUE] [smallint] NOT NULL,
	[STATUS] [tinyint] NOT NULL,
	[RESPONSECODE] [tinyint] NOT NULL,
	[NUMSATS] [tinyint] NOT NULL,
	[LATITUDE] [float] NOT NULL,
	[LONGITUDE] [float] NOT NULL,
	[GPSRESPONSETIME] [smallint] NOT NULL,
	[SPEED] [smallint] NOT NULL,
	[HEADING] [smallint] NOT NULL,
	[POSACCURACY] [int] NOT NULL,
	[IPADDRESS] [varchar](15) NOT NULL,
	[CREATED] [datetime] NOT NULL,
	[DIGITALIN] [int] NULL,
	[ANALOGIN1] [int] NULL,
	[ANALOGIN2] [int] NULL,
	[RESPONSEMSG] [varchar](512) NULL,
	[HomeViolation] [tinyint] NULL,
	[LandmarkName] [varchar](100) NULL,
	[LandmarkID] [bigint] NULL,
	[ZoneViolation] [tinyint] NULL,
	[UTCTimeTag] [datetime] NOT NULL,
	[StopStatus] [char](1) NULL,
	[StopTime] [int] NULL,
	[StopRecId] [bigint] NULL,
	[Distance] [float] NULL,
	[ASSISTANCEUSED] [char](1) NOT NULL,
	[PosSrc] [char](1) NOT NULL,
	[Street] [nvarchar](max) NULL,
	[City] [nvarchar](max) NULL,
	[State] [nvarchar](max) NULL,
	[ZipCode] [nvarchar](10) NULL,
	[Country] [nvarchar](40) NULL,
	[County] [nvarchar](40) NULL,
	[SpeedLimitViolation] [tinyint] NULL,
	[StreetSpeedLimit] [int] NULL,
	[SpeedMph] [tinyint] NULL,
	[TIMETAG] [datetime] NOT NULL,
	[IgnitionOn] [bit] NULL,
	[Attributes] [xml] NULL,
	[IdleStatus] [char](1) NULL,
	[IdleTime] [smallint] NULL,
	[IdleRecId] [bigint] NULL,
	[GeofenceViolation] [bit] NULL,
	[ModTime] [datetime] NULL,
	[CalcSpeedMph] [tinyint] NULL,
	[SpeedViolation] [bit] NULL,
	[StopTimeViolation] [bit] NULL,
	[IdleTimeViolation] [bit] NULL,
	[HardmountEventViolation] [bit] NULL,
	[IsTrueGps] [bit] NULL,
	[CloudModTime] [datetime2](4) NULL,
	[MovementViolation] [bit] NULL,
	[MovementStatus] [char](1) NULL,
	[MovementStatusTime] [int] NULL
) ON [PRIMARY] TEXTIMAGE_ON [PRIMARY]
GO

select * 
into dbo.Data_Row
from dbo.Data;

select * 
into dbo.Data_Page
from dbo.Data;
go

alter table dbo.Data add constraint PK_Data primary key clustered(CompanyId,RecId) 
alter table dbo.Data_Row add constraint PK_DataRow primary key clustered(CompanyId,RecId) with (data_compression=row);
alter table dbo.Data_Page add constraint PK_DataPage primary key clustered(CompanyId,RecId) with (data_compression=page);

select 
	s.name + '.' + t.name as [Table]
	,i.alloc_unit_type_desc
	,sum(i.page_count) as [Pages]
	,sum(i.page_count * 8 / 1024) as [Size MB]
from 
	sys.tables t join sys.schemas s on
		t.schema_id = s.schema_id
	cross apply 
		sys.dm_db_index_physical_stats
		(
			db_id() /*Database */
			,t.object_id
			,1 /* Index ID */
			,null /* Partition ID  NULL  all partitions */
			,'detailed' /* Mode */
		) i
where
	s.name = 'dbo' and t.name in ('Data','Data_Row','Data_Page')
group by 
	s.name, t.name, i.alloc_unit_type_desc
go

dbcc dropcleanbuffers
set statistics time on
select count(distinct numsats) from dbo.Data option (maxdop 1)
select count(distinct numsats) from dbo.Data_Row option (maxdop 1)
select count(distinct numsats) from dbo.Data_Page option (maxdop 1)
set statistics time off
go

set statistics time on
update dbo.Data set NumSats = NumSats + 1 option (maxdop 1)
update dbo.Data_Row set NumSats = NumSats + 1 option (maxdop 1)
update dbo.Data_Page set NumSats = NumSats + 1 option (maxdop 1)
set statistics time off
go

set statistics time on
alter index PK_Data on dbo.Data rebuild with (maxdop=1)
alter index PK_DataRow on dbo.Data_Row rebuild with (maxdop=1)
alter index PK_DataPage on dbo.Data_Page rebuild with (maxdop=1)
set statistics time off
go

update top (500) dbo.Data 
set Street = Replicate('a',16000)

update top (500) dbo.Data_Row 
set Street = Replicate('a',16000)

update top (500) dbo.Data_Page
set Street = Replicate('a',16000)
go

select 
	s.name + '.' + t.name as [Table]
	,i.alloc_unit_type_desc
	,sum(i.page_count) as [Pages]
	,sum(i.page_count * 8 / 1024) as [Size MB]
from 
	sys.tables t join sys.schemas s on
		t.schema_id = s.schema_id
	cross apply 
		sys.dm_db_index_physical_stats
		(
			db_id() /*Database */
			,t.object_id
			,1 /* Index ID */
			,null /* Partition ID  NULL  all partitions */
			,'detailed' /* Mode */
		) i
where
	s.name = 'dbo' and t.name in ('Data','Data_Row','Data_Page')
group by 
	s.name, t.name, i.alloc_unit_type_desc
go

